@@ -14,9 +14,14 @@ Requires an object on init which has the following properties:
 Copyright 2018, 2019, 2020 William W. Kimball, Jr. MBA MSIS
 """
 import sys
-from typing import Any, Dict, Generator, List, Tuple, Union
+from typing import Any, Dict, Generator, List, Set, Tuple, Union
 
-from ruamel.yaml.comments import CommentedMap
+from ruamel.yaml.comments import (
+    CommentedBase,
+    CommentedMap,
+    CommentedSet,
+    TaggedScalar
+)
 
 from yamlpath.wrappers.nodecoords import NodeCoords
 
@@ -198,20 +203,27 @@ class ConsolePrinter:
                 print(ConsolePrinter._debug_prefix_lines(
                     "{}{}".format(prefix, footer)))
 
-    @classmethod
-    def _debug_prefix_lines(cls, line: str) -> str:
+    @staticmethod
+    def _debug_prefix_lines(line: str) -> str:
         """Helper for debug."""
         return "DEBUG:  {}".format(str(line).replace("\n", "\nDEBUG:  "))
 
-    @classmethod
-    def _debug_get_anchor(cls, data: Any) -> str:
+    @staticmethod
+    def _debug_get_anchor(data: Any) -> str:
         """Helper for debug."""
         return ("&{}".format(data.anchor.value)
                 if hasattr(data, "anchor") and data.anchor.value is not None
                 else "")
 
-    @classmethod
-    def _debug_dump(cls, data: Any, **kwargs) -> Generator[str, None, None]:
+    @staticmethod
+    def _debug_get_tag(data: Any) -> str:
+        """Helper for debug."""
+        return str(data.tag.value
+                if hasattr(data, "tag") and data.tag.value is not None
+                else "")
+
+    @staticmethod
+    def _debug_dump(data: Any, **kwargs) -> Generator[str, None, None]:
         """Helper for debug."""
         prefix = kwargs.pop("prefix", "")
         if isinstance(data, dict):
@@ -219,7 +231,7 @@ class ConsolePrinter:
                 data, prefix=prefix, **kwargs
             ):
                 yield line
-        elif isinstance(data, (list, tuple)):
+        elif isinstance(data, (list, set, tuple)):
             for line in ConsolePrinter._debug_list(
                 data, prefix=prefix, **kwargs
             ):
@@ -232,24 +244,41 @@ class ConsolePrinter:
         else:
             yield ConsolePrinter._debug_scalar(data, prefix=prefix, **kwargs)
 
-    @classmethod
-    def _debug_scalar(cls, data: Any, **kwargs) -> str:
+    @staticmethod
+    def _debug_scalar(data: Any, **kwargs) -> str:
         """Helper for debug."""
         prefix = kwargs.pop("prefix", "")
         print_anchor = kwargs.pop("print_anchor", True)
+        print_tag = kwargs.pop("print_tag", True)
         print_type = kwargs.pop("print_type", False)
+        dtype = type(data) if print_type else ""
+        anchor_prefix = ""
+        print_prefix = prefix
 
         if print_anchor:
             anchor = ConsolePrinter._debug_get_anchor(data)
             if anchor:
-                prefix += "({})".format(anchor)
+                anchor_prefix = "({})".format(anchor)
 
+        if print_tag:
+            if isinstance(data, TaggedScalar):
+                tag_prefix = "{}{}<{}>".format(
+                    print_prefix, anchor_prefix, data.tag.value)
+                return ConsolePrinter._debug_scalar(
+                        data.value, prefix=tag_prefix,
+                        print_anchor=False, print_tag=False, print_type=True)
+
+        # The "true" type of the value is nested in TaggedScalar.value
+        if isinstance(data, TaggedScalar):
+            dtype = "{}({})".format(dtype, type(data.value))
+
+        print_prefix += anchor_prefix
         return ConsolePrinter._debug_prefix_lines(
-            "{}{}{}".format(prefix, data, (type(data) if print_type else "")))
+            "{}{}{}".format(print_prefix, data, dtype))
 
-    @classmethod
+    @staticmethod
     def _debug_node_coord(
-        cls, data: NodeCoords, **kwargs
+        data: NodeCoords, **kwargs
     ) -> Generator[str, None, None]:
         """Helper method for debug."""
         prefix = kwargs.pop("prefix", "")
@@ -274,28 +303,38 @@ class ConsolePrinter:
         ):
             yield line
 
-    @classmethod
+    @staticmethod
     def _debug_list(
-        cls, data: Union[List[Any], Tuple[Any, ...]], **kwargs
+        data: Union[List[Any], Tuple[Any, ...], Set[Any]], **kwargs
     ) -> Generator[str, None, None]:
         """Helper for debug."""
         prefix = kwargs.pop("prefix", "")
+        print_tag = kwargs.pop("print_tag", True)
+
+        if (print_tag
+            and isinstance(data, (CommentedBase, CommentedSet))
+            and hasattr(data, "tag")
+            and data.tag.value
+        ):
+            prefix += "<{}>".format(data.tag.value)
+
         for idx, ele in enumerate(data):
             ele_prefix = "{}[{}]".format(prefix, idx)
-            if isinstance(ele, dict):
-                for line in ConsolePrinter._debug_dict(ele, prefix=ele_prefix):
-                    yield line
-            elif isinstance(ele, list):
-                for line in ConsolePrinter._debug_list(ele, prefix=ele_prefix):
-                    yield line
-            else:
-                for line in ConsolePrinter._debug_dump(
-                    ele, prefix=ele_prefix, print_type=True
-                ):
-                    yield line
 
-    @classmethod
-    def _debug_get_kv_anchors(cls, key: Any, value: Any) -> str:
+            # if (isinstance(data, (CommentedBase, CommentedSet))
+            #     and hasattr(ele, "tag")
+            #     and ele.tag.value
+            # ):
+            #     ele_prefix += "<{}>".format(ele.tag.value)
+
+            for line in ConsolePrinter._debug_dump(
+                ele, prefix=ele_prefix, print_anchor=True, print_tag=True,
+                print_type=True
+            ):
+                yield line
+
+    @staticmethod
+    def _debug_get_kv_anchors(key: Any, value: Any) -> str:
         """Helper for debug."""
         key_anchor = ConsolePrinter._debug_get_anchor(key)
         val_anchor = ConsolePrinter._debug_get_anchor(value)
@@ -308,9 +347,23 @@ class ConsolePrinter:
             display_anchor = "(_,{})".format(val_anchor)
         return display_anchor
 
-    @classmethod
+    @staticmethod
+    def _debug_get_kv_tags(key: Any, value: Any) -> str:
+        """Helper for debug."""
+        key_tag = ConsolePrinter._debug_get_tag(key)
+        val_tag = ConsolePrinter._debug_get_tag(value)
+        display_tag = ""
+        if key_tag and val_tag:
+            display_tag = "<{},{}>".format(key_tag, val_tag)
+        elif key_tag:
+            display_tag = "<{},_>".format(key_tag)
+        elif val_tag:
+            display_tag = "<_,{}>".format(val_tag)
+        return display_tag
+
+    @staticmethod
     def _debug_dict(
-        cls, data: Union[Dict, CommentedMap], **kwargs
+        data: Union[Dict, CommentedMap], **kwargs
     ) -> Generator[str, None, None]:
         """Helper for debug."""
         prefix = kwargs.pop("prefix", "")
@@ -328,14 +381,12 @@ class ConsolePrinter:
                            if key in local_keys
                            else "<<:{}:>>".format(key))
             display_anchor = ConsolePrinter._debug_get_kv_anchors(key, val)
-            kv_prefix = "{}[{}]{}".format(prefix, display_key, display_anchor)
+            display_tag = ConsolePrinter._debug_get_kv_tags(key, val)
+            kv_prefix = "{}[{}]{}{}".format(
+                prefix, display_key, display_anchor, display_tag)
 
-            if isinstance(val, dict):
-                for line in ConsolePrinter._debug_dict(val, prefix=kv_prefix):
-                    yield "{}".format(line)
-            elif isinstance(val, list):
-                for line in ConsolePrinter._debug_list(val, prefix=kv_prefix):
-                    yield "{}".format(line)
-            else:
-                yield ConsolePrinter._debug_scalar(
-                    val, prefix=kv_prefix, print_type=True, print_anchor=False)
+            for line in ConsolePrinter._debug_dump(
+                val, prefix=kv_prefix, print_type=True, print_tag=False,
+                print_anchor=False
+            ):
+                yield line
